import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Home } from "@mui/icons-material";
import { Grid, Button, Box, Typography } from "@mui/material";
import { useState } from "react";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../i18n";
/**
 *
 * @param {NerdAppsMenuProps} props Component props
 * @returns {ReactElement} NerdAppsMenu
 */
export default function NerdAppsMenu(props) {
    const { apps, homePath, appIconHeight, appIconWidth } = props;
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    return (_jsxs(Grid, { container: true, padding: "10px", children: [apps.map((app, i) => {
                return (_jsx(Grid, { item: true, xs: 4, sm: 3, children: _jsx(AppIcon, { iconHeigt: appIconHeight, iconWidth: appIconWidth, app: app }) }, app.name?.concat(i.toString())));
            }), _jsx(Grid, { item: true, xs: 12, mt: 2, textAlign: "center", children: homePath && (_jsx(Button, { href: homePath, variant: "outlined", startIcon: _jsx(Home, {}), children: t("components:nerdAppsMenu.home") })) })] }));
}
/**
 *
 * @param {object} props Component props
 * @param {App} props.app App to render
 * @param {number | undefined} props.iconWidth App icon width
 * @param {number | undefined} props.iconHeigt App to render
 * @returns {ReactElement} AppIcon
 */
function AppIcon(props) {
    const { app, iconHeigt, iconWidth } = props;
    const _iconHeigt = iconHeigt || 40;
    const _iconWidth = iconWidth || 40;
    const [hover, setHover] = useState(false);
    const transition = "ease-in";
    const { t } = useTranslation("components", { i18n: nerdUII18n });
    return (_jsxs(Box, { sx: {
            display: "flex",
            flexDirection: "column",
            alignContent: "center",
            alignItems: "center",
        }, onMouseEnter: () => setHover(true), onMouseLeave: () => setHover(false), children: [app.url ? (_jsx("a", { href: `https://${app.url}`, style: {
                    textDecoration: "none",
                }, children: app.icon ? (_jsx("img", { src: app.icon, alt: t("nerdAppsMenu.app_icon_alt"), width: _iconWidth, height: _iconHeigt, style: {
                        borderRadius: "20%",
                        boxShadow: hover
                            ? "0 5px 18px 0 rgba(0,0,0,.3)"
                            : "0 5px 28px 0 rgba(0,0,0,.1)",
                        transition: `box-shadow .2s ${transition}`,
                    } })) : (_jsx(Box, { sx: {
                        backgroundColor: app.color,
                        marginBottom: "5px",
                        width: `${_iconWidth}px`,
                        height: `${_iconHeigt}px`,
                        textAlign: "center",
                        border: `solid ${40 * 0.05}px ${app.color}`,
                        background: "#424242",
                        color: app.color,
                        fontFamily: "Futura",
                        fontSize: _iconHeigt * 0.5,
                        borderTopRightRadius: 40 * 0.5,
                        borderBottomLeftRadius: 40 * 0.5,
                        boxShadow: hover
                            ? "0 5px 18px 0 rgba(0,0,0,.3)"
                            : "0 5px 28px 0 rgba(0,0,0,.1)",
                        transition: `box-shadow .2s ${transition}`,
                    }, children: _jsx(Grid, { container: true, justifyContent: "center", alignItems: "center", style: { height: 35 }, children: app.name && _jsx(Grid, { item: true, children: `${app.name[0]}${app.name[1]}` }) }) })) })) : null, _jsx(Typography, { fontWeight: "300", fontSize: "14px", textAlign: "center", children: app.name })] }));
}
