import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useCallback, useEffect, useState, useRef, useMemo, } from "react";
import { DataGridPro, GridToolbarContainer, GridToolbarFilterButton, GridLinkOperator, GridActionsCellItem, GridToolbarQuickFilter, } from "@mui/x-data-grid-pro";
import CustomPagination from "../nerdPagination/nerdPagination";
import { Button, IconButton, Tooltip } from "@mui/material";
import { Delete, SaveAlt } from "@mui/icons-material";
import { operatorsByType } from "./operatorsByType";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../i18n";
import { useSearchParams } from "react-router-dom";
/**
 * NerdDataGrid component.
 *
 * @param {propsType} props NerdDataGrid props.
 * @returns {ReactElement} DataGrid wrapper with server-side filters.
 */
export default function NerdDataGrid(props) {
    const { columnsVisibility, columns, getRows, culumnsMatchingFilterName, hidePagination, hideFilters, defaultSelectRowID, createNewButton, requestPending, initialRowsPerPage, editable, updateRow, deleteRow, deleteSelectedRows, } = props;
    const rows = props.rows;
    const rowPageOptions = [10, 50, 100];
    const [searchParams, setSearchParams] = useSearchParams();
    const limit = searchParams.get("limit");
    const offset = searchParams.get("offset");
    const filters = searchParams.get("filters");
    const [rowsPerPage, setRowsPerPage] = useState(initialRowsPerPage ? initialRowsPerPage : 10);
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    const [page, setPage] = useState(0);
    const [selectPages, setSelectPages] = useState([0]);
    const [disabledNext, setDisabledNext] = useState(false);
    const [rowCount, setRowCount] = useState(undefined);
    const [rowCountState, setRowCountState] = useState(rowCount || 0);
    const [filterModel, setFilterModel] = useState({
        items: [],
        linkOperator: GridLinkOperator.And,
    });
    const [selectionModel, setSelectionModel] = useState(defaultSelectRowID &&
        rows.filter((row) => row["id"] === defaultSelectRowID).length > 0
        ? [defaultSelectRowID]
        : []);
    const prevSelectionModel = useRef(defaultSelectRowID ? [defaultSelectRowID] : []);
    useEffect(() => {
        if (getRows) {
            getRows({
                limit: limit || initialRowsPerPage?.toString() || "10",
                offset: offset || "0",
                filters: filters || "",
            });
        }
    }, []);
    // Check if API call ends and close loading
    useEffect(() => {
        setRowCount(rows.length);
        //restore selection model after page change
        setSelectionModel(prevSelectionModel.current);
        setCustomPagination();
    }, [rows]);
    useEffect(() => {
        try {
            if (culumnsMatchingFilterName &&
                filters &&
                culumnsMatchingFilterName.size > 0) {
                const filtersObject = JSON.parse(filters);
                const filterModelTmp = {
                    items: [],
                    linkOperator: GridLinkOperator.And,
                };
                Array.isArray(filtersObject) &&
                    filtersObject.forEach((filter, i) => {
                        let columnField = "";
                        for (const [key, value] of culumnsMatchingFilterName.entries()) {
                            if (value === filter.name)
                                columnField = key;
                        }
                        filterModelTmp.items.push({
                            id: i + 1,
                            columnField,
                            value: filter.value,
                            operatorValue: filter.comparison,
                        });
                    });
                setFilterModel(filterModelTmp);
            }
        }
        catch (error) {
            return;
        }
    }, [culumnsMatchingFilterName]);
    // Set URL query
    useEffect(() => {
        if (offset && limit) {
            if (Number(offset) % Number(limit) !== 0 ||
                !rowPageOptions.includes(Number(limit))) {
                const limit = initialRowsPerPage?.toString() || "10";
                setPage(1);
                setRowsPerPage(Number(limit));
                setSearchParams({
                    limit,
                    offset: "0",
                    ...(filters ? { filters: filters } : {}),
                });
            }
            else {
                setPage(Number(offset) / Number(limit) + 1);
                setRowsPerPage(Number(limit));
            }
        }
        else {
            setPage(1);
            setRowsPerPage(initialRowsPerPage || 10);
        }
    }, [limit, offset]);
    useEffect(() => {
        setRowCountState((prevRowCountState) => rowCount !== undefined ? rowCount : prevRowCountState);
    }, [rowCount, setRowCountState]);
    /**
     * Custom MUI Grid Toolbar
     *
     * @returns {React.ReactElement} Mui Grid Toolbar
     */
    const CustomToolbar = useCallback(() => {
        return (_jsxs(GridToolbarContainer, { children: [!hideFilters && _jsx(GridToolbarFilterButton, {}), props.apiExportEndpoint && (_jsx(Button, { sx: { fontSize: "0.8rem" }, variant: "text", startIcon: _jsx(SaveAlt, {}), onClick: () => {
                        const a = encodeURI(`${props.apiExportEndpoint}${filters ? `?filters=${filters}` : ""}`);
                        window.open(a, "_blank", "noopener,noreferrer");
                    }, children: t("components:nerdDataGrid.export") })), createNewButton, editable && (_jsx(Tooltip, { title: t("components:nerdDataGrid.delete_rows"), children: _jsx(IconButton, { color: "primary", onClick: () => {
                            const selectedRows = rows.filter((row) => selectionModel.find((id) => row.id === id));
                            deleteSelectedRows?.(selectedRows.map((row) => {
                                return row.id;
                            }));
                        }, children: _jsx(Delete, {}) }) })), props.componentsProps?.toolbar?.showQuickFilter ? (_jsx(GridToolbarQuickFilter, { sx: {
                        ml: "auto",
                    } })) : null] }));
    }, []);
    const setCustomPagination = useCallback(() => {
        setSelectPages((prevState) => {
            const toto = [
                ...Array.from(new Set(prevState)),
                ...Array(page)
                    .fill(0)
                    .map((_e, i) => i + 1),
            ];
            return toto;
        });
    }, [limit, offset, page]);
    const onFilterChange = useCallback((newFilterModel) => {
        if (culumnsMatchingFilterName) {
            const filters = [];
            newFilterModel.items.forEach((item) => {
                const searchMetadataName = culumnsMatchingFilterName.get(item.columnField);
                if (item.operatorValue &&
                    item.value !== undefined &&
                    item.value !== "" &&
                    searchMetadataName) {
                    filters.push({
                        name: searchMetadataName,
                        comparison: item.operatorValue,
                        value: item.value,
                    });
                }
            });
            setPage(1); // Reset page to 1
            setSelectPages([0]); // Reset select pages
            setSearchParams({
                limit: limit || initialRowsPerPage?.toString() || "10",
                offset: "0",
                ...(filters.length > 0 ? { filters: JSON.stringify(filters) } : {}),
            });
            if (getRows) {
                setRowCount(undefined);
                getRows({
                    limit: limit || initialRowsPerPage?.toString() || "10",
                    offset: offset || "0",
                    ...(filters.length > 0 ? { filters: JSON.stringify(filters) } : {}),
                });
            }
        }
        setFilterModel(newFilterModel);
    }, [culumnsMatchingFilterName, limit]);
    /**
     * Set new page value and change the query object
     *
     * @param {React.MouseEvent<HTMLButtonElement> | null} _event ChangePage
     * @param {number} newPage New page index
     */
    function handleChangePage(_event, newPage) {
        prevSelectionModel.current = selectionModel;
        setPage(newPage);
        const _limit = limit || initialRowsPerPage?.toString() || "10";
        if (newPage > 0) {
            setSearchParams({
                limit: _limit,
                offset: ((newPage - 1) * Number(_limit)).toString(),
                ...(filters ? { filters: filters } : {}),
            });
            if (getRows) {
                setRowCount(undefined);
                getRows({
                    limit: _limit,
                    offset: ((newPage - 1) * Number(_limit)).toString(),
                    ...(filters ? { filters: filters } : {}),
                });
            }
        }
    }
    /**
     * @param {ChangeEvent<HTMLInputElement | HTMLTextAreaElement>} event - new rows page number selected
     */
    function handleChangeRowsPerPage(event) {
        const newPage = event.target.value.toString();
        setPage(1); // Reset page to 1
        setSelectPages([0]); // Reset select pages
        // Set new query object
        setSearchParams({
            limit: newPage,
            offset: "0",
            ...(filters ? { filters: filters } : {}),
        });
        if (getRows) {
            setRowCount(undefined);
            getRows({
                limit: newPage,
                offset: "0",
                ...(filters ? { filters: filters } : {}),
            });
        }
    }
    /**
     *
     * @param {GridSelectionModel} ids Selected rows
     */
    function onSelectionModelChange(ids) {
        const selectedIDs = new Set(ids);
        const selectedRows = rows.filter((row) => selectedIDs.has(row.id));
        const prevSelectedIDs = new Set(prevSelectionModel.current);
        //Don't call onSelect if page changed
        if (ids.length === 0) {
            if (rows.filter((row) => prevSelectedIDs.has(row.id)).length > 0) {
                props.onSelect?.(selectedRows);
            }
        }
        else {
            props.onSelect?.(selectedRows);
        }
        setSelectionModel(ids);
    }
    const handleDeleteClick = (id) => () => {
        deleteRow?.(Number(id));
    };
    const processRowUpdate = (newRow) => {
        const updatedRow = { ...newRow, isNew: false };
        updateRow?.(updatedRow);
        return updatedRow;
    };
    const columnsWithOperators = useMemo(() => {
        const columnsWithOperators = columns.map((col) => {
            return {
                ...col,
                filterOperators: operatorsByType(col.type),
            };
        });
        if (editable) {
            columnsWithOperators.push({
                field: "actions",
                type: "actions",
                headerName: "Actions",
                width: 100,
                cellClassName: "actions",
                getActions: ({ id }) => {
                    return [
                        _jsx(GridActionsCellItem, { icon: _jsx(Delete, {}), label: t("global.delete"), onClick: handleDeleteClick(id), onKeyUp: (event) => {
                                if (event.key === "Enter")
                                    handleDeleteClick(id);
                            } }, "GridActionDelete"),
                    ];
                },
            });
        }
        return columnsWithOperators;
    }, [columns]);
    return (_jsx(DataGridPro, { ...props, pageSize: rowsPerPage, rowsPerPageOptions: rowPageOptions, onStateChange: (state) => {
            const { pagination: { rowCount, pageSize }, } = state;
            setDisabledNext(rowCount < pageSize);
        }, pagination: true, paginationMode: "server", rowCount: rowCountState, hideFooterPagination: hidePagination, components: {
            Toolbar: CustomToolbar,
            Pagination: CustomPagination,
            ...props.components,
        }, componentsProps: {
            ...props.componentsProps,
            filterPanel: {
                ...props.componentsProps?.filterPanel,
                linkOperators: ["and"],
            },
            pagination: {
                ...props.componentsProps?.pagination,
                rowsPerPage,
                handleChangeRowsPerPage,
                rowPageOptions,
                handleChangePage,
                page,
                selectPages,
                disabledNext,
            },
        }, onSelectionModelChange: onSelectionModelChange, rows: rows, columns: columnsWithOperators, loading: requestPending, filterMode: props.componentsProps?.toolbar?.showQuickFilter ? "client" : "server", onFilterModelChange: onFilterChange, initialState: {
            columns: {
                columnVisibilityModel: columnsVisibility,
            },
        }, filterModel: filterModel, selectionModel: selectionModel, ...(editable && {
            experimentalFeatures: {
                newEditingApi: true,
            },
            processRowUpdate: processRowUpdate,
            checkboxSelection: true,
            disableSelectionOnClick: true,
        }) }));
}
