import { GridFilterInputDate, GridFilterInputValue, GridFilterInputSingleSelect, } from "@mui/x-data-grid-pro";
import BoolOperatorInput from "./customOperatorInput/boolTypeInput";
const OPERATORS = {
    eq: {
        label: "Equal",
        value: "eq",
    },
    neq: {
        label: "Not Equal",
        value: "neq",
    },
    gt: {
        label: "Greater Than",
        value: "gt",
    },
    lt: {
        label: "Less Than",
        value: "lt",
    },
    gte: {
        label: "Greater or Equal Than",
        value: "gte",
    },
    lte: {
        label: "Less or Equal Than",
        value: "lte",
    },
    like: {
        label: "Like",
        value: "like",
    },
};
/**
 * Return DataGrid filter operators for a given column type.
 *
 * @param {string | undefined} type Type of the column.
 * @returns {Array<GridFilterOperator>} Return filter operators.
 */
export function operatorsByType(type) {
    let input;
    let inputProps = {};
    let operatorsKey = [];
    switch (type) {
        case "date":
            input = GridFilterInputDate;
            inputProps = { type: "date" };
            operatorsKey = ["eq", "neq", "gt", "lt", "gte", "lte"];
            break;
        case "dateTime":
            input = GridFilterInputDate;
            inputProps = { type: "datetime-local" };
            operatorsKey = ["eq", "neq", "gt", "lt", "gte", "lte"];
            break;
        case "dateLike":
            input = GridFilterInputDate;
            inputProps = { type: "date" };
            operatorsKey = ["gt", "lt", "gte", "like"];
            break;
        case "boolean":
            input = BoolOperatorInput;
            operatorsKey = ["eq"];
            break;
        case "number":
            input = GridFilterInputValue;
            inputProps = { type: "number" };
            operatorsKey = ["eq", "neq", "gt", "lt", "gte", "lte", "like"];
            break;
        case "singleSelect":
            input = GridFilterInputSingleSelect;
            operatorsKey = ["eq", "neq"];
            break;
        case "actions":
            input = undefined;
            break;
        case "string":
            input = GridFilterInputValue;
            operatorsKey = ["eq", "neq", "like"];
            break;
        default:
            input = GridFilterInputValue;
            operatorsKey = ["eq", "neq", "gt", "lt", "gte", "lte", "like"];
            break;
    }
    return createOperators(operatorsKey, input, inputProps);
}
/**
 *
 * @param {string[]} operatorsKey Operators key
 * @param {JSXElementConstructor<GridFilterInputValueProps>} input Grid Filter Input
 * @param {Record<string, unknown>} inputProps Grid Filter Input Props
 * @returns {GridFilterOperator<any, any, any>[]} Grid Filter Operators
 */
function createOperators(operatorsKey, input, inputProps) {
    const operators = [];
    operatorsKey.forEach((key) => {
        const val = OPERATORS[key];
        operators.push({
            label: val.label,
            value: val.value,
            getApplyFilterFn: () => {
                //ServerSide filter, so we don't need filter function here.
                return () => {
                    return true;
                };
            },
            InputComponent: input,
            InputComponentProps: inputProps,
        });
    });
    return operators;
}
