import { jsx as _jsx } from "react/jsx-runtime";
import { WidgetType } from "@codemirror/view";
import { createRoot } from "react-dom/client";
import { unified } from "unified";
import remarkDirective from "remark-directive";
import remarkParse from "remark-parse";
import remarkGfm from "remark-gfm";
import remarkHtml from "remark-html";
import { ReactMarkdown } from "react-markdown/lib/react-markdown";
import { Button } from "@mui/material";
import remarkDirectiveRehype from "./remarkDirectiveRehype";
export class BulletListWidget extends WidgetType {
    id;
    text;
    constructor(id, text) {
        super();
        this.id = id;
        this.text = text;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM() {
        const parent = document.createElement("span");
        parent.innerHTML = "&#x2022;";
        return parent;
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
export class TaskWidget extends WidgetType {
    id;
    text;
    constructor(id, text) {
        super();
        this.id = id;
        this.text = text;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM() {
        const parent = document.createElement("span");
        const checkbox = document.createElement("input");
        checkbox.type = "checkbox";
        checkbox.disabled = true;
        if (this.text.includes("x") || this.text.includes("X")) {
            checkbox.checked = true;
        }
        parent.append(checkbox);
        return parent;
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
export class MarkdownWidget extends WidgetType {
    id;
    text;
    constructor(id, text) {
        super();
        this.id = id;
        this.text = text;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM() {
        const parent = document.createElement("span");
        const processor = unified().use(remarkParse).use(remarkGfm).use(remarkHtml);
        parent.innerHTML = String(processor.processSync(this.text));
        const child = parent.firstChild
            ? parent.firstChild
            : parent.firstChild;
        if (child) {
            child.style.margin = "0";
            child.style.display = "inline-block";
            if (child.tagName == "UL" && this.text.match(/- \[( |x|X)\]/))
                child.style.listStyleType = "none";
        }
        return parent;
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
export class MarkdownTextDirectiveWidget extends WidgetType {
    id;
    text;
    constructor(id, text) {
        super();
        this.id = id;
        this.text = text;
    }
    eq(other) {
        return other.id == this.id;
    }
    toDOM() {
        try {
            const parent = document.createElement("span");
            const el = document.createElement("span");
            parent.appendChild(el);
            parent.style.display = "inline-block";
            el.style.display = "inline-block";
            const root = createRoot(el);
            root.render(_jsx(ReactMarkdown, { remarkPlugins: [remarkGfm, remarkDirective, remarkDirectiveRehype], components: { "button-component": Button }, children: this.text }));
            return parent.appendChild(el);
        }
        catch (e) {
            console.error(e);
            return document.createElement("div");
        }
    }
    ignoreEvent() {
        return false;
    }
    destroy(dom) {
        dom.remove();
    }
}
